# Base Classes for NeoBoard

from AccessControl import ClassSecurityInfo, getSecurityManager
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from OFS.DTMLMethod import DTMLMethod
from OFS.SimpleItem import SimpleItem
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from StructuredText import HTMLClass, DocumentClass, ST
from DateTime.DateTime import DateTime
from NB_Utils import stripTags, cutString

Document = DocumentClass.DocumentClass()
HTMLNG = HTMLClass.HTMLClass()

class NB_BaseItem(SimpleItem, CatalogAware, PropertyManager):
	"""
		NeoBoardBase Item Class
	"""
	_renderables = ('RAW', 'HTML','STX', 'DTML')	
	
	def getSelf(self):
		"""
			Returns inner self if wrapped, otherwise self
		"""
		if hasattr(self, 'aq_chain'):
			return self.aq_chain[0]
		else:
			return self

	def getParent(self):
		"""
			Returns parent object
		"""
		if hasattr(self, 'aq_inner'):	
			return self.aq_inner.aq_parent
		else:
			if hasattr(self, 'aq_parent'):
				return self.aq_parent
			else:
				return None

	def getTitle(self):
		"""
			Returns title
		"""
		
		return self.title_or_id()

	def getMenuTitle(self, trim_title=1, chunk_size=15):
		"""
			Returns title to appear on Menu
		"""

		if hasattr(self, 'menu_title'):
			if trim_title:
				return cutString(self.menu_title, chunk=chunk_size)
			else:
				return self.menu_title
		else:
			if trim_title:
				return cutString(self.getTitle, chunk=chunk_size)
			else:
				return self.getTitle()

	def getDescription(self):
		"""
			Returns description
		"""	
		
		if hasattr(self.getSelf(), 'description'):
			return self.description
		else:
			return ''


	def setEnabled(self, mode=1):
		"""
			Enable or disable the object
		"""
		
		self.enabled = mode

	def isEnabled(self):
		"""
			Tells if the object is enabled
		"""
		
		return self.enabled

	def isViewable(self):
		"""
			Tells if the object is publicly viewable
		"""
		
		if not self.isEnabled():
			return 0
		
		checkPermission=getSecurityManager().checkPermission
		if checkPermission('Access contents information', self) and checkPermission('View', self):
			return 1
		else:
			return 0

	def isEditable(self):
		"""
			Tells if the authenticated user is authorized to edit this object
		"""
		
		if not self.isViewable():
			return 0
		
		checkPermission=getSecurityManager().checkPermission
		if checkPermission('View management screens', self):
			return 1
		else:
			return 0

	def selfUpgrade(self):	
		"""
			Upgrade Old Class Instances
		"""
		pass					

class NB_BaseContentItem(NB_BaseItem):
	"""
	NeoBoardBase Content Item Class
	"""
	
	_default_format = 'HTML'
	_default_charset = 'ISO-8859-1'
	_allowed_charsets = ['ISO-8859-1', 'EUC-KR', 'KSC_5601',	
			'UTF-8', 'UTF-16', 'US-ASCII', 
			'ISO-8859-2', 'ISO-8859-3', 'ISO-8859-4', 
			'ISO-8859-5', 'ISO-8859-6', 'ISO-8859-7', 'ISO-8859-8', 
			'ISO-8859-9', 'ISO-2022-JP', 'ISO-2022-JP-2', 
			'ISO-2022-KR', 'windows-1252', 'GB2312', 'HZ-GB-2312',
			'SHIFT_JIS', 'JIS_X0208', 'BIG5', 'KOI8-R',
			]
	
	_default_language = 'en-US'
	
	_allowed_languages = {'en-US':'American English', 
			'en-GB':'British English', 
			'en-CA': 'Canadian English',
			'en-AU': 'Austrailian English',
			'kr': 'Korean',
			'ja': 'Japanese',
			'zh': 'Chinese',			
			'fr': 'French',
			'de': 'German',
			'it': 'Italian',
			'es': 'Spanish',
			}
	
	def __init__(self):
		self.setContentCacheable()

   	def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
   		return self.getRenderedContent()

	def PrincipiaSearchSource(self, REQUEST=None):
    		"""
    			Supports for full text search via ZCatalog indexing
    		"""
		delimiter = ' '
		return delimiter.join((self.getTitle(), self.getDescription(), self.getKeywordsAsString(), self.getRenderedContent(),))
   	   		   		
	
	def getDefaultCharset(self):
		"""
			Returns the default charset of the content
		"""
		return self._default_charset
		
	def getDefaultFormat(self):
		"""
			Returns the default format of the content
		"""
		return self._default_format

	def getDefaultLanguage(self):
		"""
			Returns the default language of the content
		"""
		return self._default_language

	def getAllowedCharsets_NB(self):
		"""
			Returns the list of allowed charsets
		"""
		
		return self._allowed_charsets

	def getAllowedLanguages_NB(self):
		"""
			Returns allowed languages as a dictionary
		"""
		return self._allowed_languages

	def getCreationDateTime(self):
		"""
			Returns Creation DateTime
		"""	
		if hasattr(self, 'creation_date') and self.creation_date is not None:
			return self.creation_date.strftime('%Y-%m-%d %H:%M:%S')
		else:
			return ''

	def getEffectiveDateTime(self):
		"""
			Returns Effective DateTime
		"""	
		if hasattr(self, 'effective_date')  and self.effective_date is not None:
			return self.effective_date.strftime('%Y-%m-%d %H:%M:%S')
		else:
			return ''

	def getExpireDateTime(self):
		"""
			Returns Expire DateTime
		"""	
		if hasattr(self, 'expire_date')  and self.expire_date is not None:
			return self.expire_date.strftime('%Y-%m-%d %H:%M:%S')
		else:
			return ''

	def getModificationDateTime(self):
		return self.bobobase_modification_time().strftime('%Y-%m-%d %H:%M:%S')

	def getCreationDate(self):
		if hasattr(self, 'creation_date') and self.creation_date is not None:
			return self.creation_date.strftime('%Y-%m-%d')
		else:
			return ''

	def getEffectiveDate(self):
		if hasattr(self, 'effective_date')  and self.effective_date is not None:
			return self.effective_date.strftime('%Y-%m-%d')
		else:
			return ''

	def getExpireDate(self):
		if hasattr(self, 'expire_date')  and self.expire_date:
			return self.expire_date.strftime('%Y-%m-%d')
		else:
			return ''

	def getModificationDate(self):
		return self.bobobase_modification_time().strftime('%Y-%m-%d')

	def isEnabled(self):
		if not self.isExpired() and self.isEffective_NB() and NB_BaseContentItem.inheritedAttribute("isEnabled")(self):
			return 1
		else:
			return 0
	
	def isExpired(self):
		if hasattr(self, 'expire_date'):
			if not self.expire_date:
				return 0
			else:
				return not self.expire_date.isFuture()
		else:
			return 0

	def isEffective_NB(self):
		if hasattr(self, 'effective_date') and self.effective_date is not None:
			return not self.effective_date.isFuture()
		else:
			return 1

	def getCharset(self):
		"""
			Returns the charset of the content
		"""	
		if hasattr(self, 'charset'):
			if not self.charset in self.getAllowedCharsets_NB():
				return self.getDefaultCharset()
			return self.charset
		else:
			return self.getDefaultCharset()

	def getFormats(self):
		"""
			Returns the format(s) of the content
		"""		
		if hasattr(self, 'formats'):
			return self.formats
		else:
			return (self.getDefaultFormat(),)

	def getLanguage(self):
		"""
			Returns the language the content is written in
		"""		
		if hasattr(self, 'language'):
			if not self.language in self.getAllowedLanguages_NB().keys():
				return self.getDefaultLanguage()
			return self.language
		else:
			return self.getDefaultLanguage()
			
	def setContentCacheable(self, mode=1):
		"""
			Sets Content Caching option
		"""
		
		self.content_cacheable = mode
		
	def isContentCacheable(self):
		"""
 			Check to see if the object is cacheable		
		"""
		if hasattr(self.getSelf(), 'content_cacheable'):
			return self.content_cacheable
		
		else:
			return 0
	
	def setRawContent(self, content):
		"""
			Sets raw content
		"""

		self.raw = content		

	def getRawContent(self):
		"""
			Return raw content
		"""
		
		if hasattr(self.getSelf(), 'raw'):
			return self.raw
		else:
			return ''


	def getRenderables(self):
		"""
			Returns a tuple of renderable formats: HTML, DTML, STX, etc
		"""
	
		return self._renderables

	def getRenderedContent(self, REQUEST=None):
		"""
			Returns the content after rendering: first check to see if the content is cached
		"""
		
		if self.isContentCacheable():
			if not hasattr(self.getSelf(), '_rendered_content'):
				if not self.updateContent():
					return ''
			
			return self._rendered_content
		else:
			if not self.updateContent():
					return ''
			
			return self._rendered_content
			

	def updateContent(self):
		"""
			Re-renders the content
		"""
		
		if not hasattr(self.getSelf(), 'raw'):
			return 0
		
		self._renderContent()

		return 1

	def _renderContent(self):	
		"""
			Renders the content
		"""
		
		content = self.getRawContent()
		
		if 'DTML' in self.getRenderables() and 'DTML' in self.getFormats():
			dtml_doc = DTMLMethod(content)
			content = apply(dtml_doc.__call__, (self, self.REQUEST, self.REQUEST.RESPONSE))
		
		if 'STX' in self.getRenderables()  and 'STX' in self.getFormats():
			content = ST.Basic(content)
			content = Document(content)
			content = HTMLNG(content, header=0)
			
		if 'HTML' not in self.getRenderables() or 'HTML' not in self.getFormats():  
			content = stripTags(content)
			
		self._rendered_content = content

	def getSize(self):
		return self.get_size()
		
	def PUT(self, REQUEST, RESPONSE):
		""" 
			Handles FTP/WebDav PUT requests 
		"""

	        self.dav__init(REQUEST, RESPONSE)
	        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
	        content = REQUEST['BODY']
	        self.setRawContent(content.read())
	        RESPONSE.setStatus(204)
	        return RESPONSE

	manage_FTPput = PUT
	manage_FTPget = getRawContent
		
	def manage_afterClone(self, item):
		"""
			Called when Copy-paste-operations performed.
		"""
		
		NB_BaseContentItem.inheritedAttribute("manage_afterClone")(self, item)
		
	def manage_afterAdd(self, item, container):
		""" 
			Called when objects get added, moved, or renamed.
		"""
			
		NB_BaseContentItem.inheritedAttribute("manage_afterAdd")(self, item, container)
	
	def manage_beforeDelete(self, item, container):
		""" 
			Called when objects get moved, cut-and-pasted, renamed, or deleted.
		"""		
		NB_BaseContentItem.inheritedAttribute("manage_beforeDelete")(self, item, container)

		
class NB_BaseContentDocument(NB_BaseContentItem, DTMLMethod):
	"""
	NeoBoardBase Content Document Class
	"""
	
        manage_options=(
                {'label':'Contents',   'action':'manage_main'},
                {'label':'View',       'action':''},
                {'label':'Properties', 'action':'manage_editProperties'},
                {'label':'Security',   'action':'manage_access'},
                {'label':'Ownership',  'action':'manage_owner'},
                {'label':'Undo',       'action':'manage_UndoForm'},
        )		

	def setRawContent(self, content):
		"""
			Sets raw content
		"""

		self.munge(content)

	def manage_edit(self,data,title,SUBMIT='Change',dtpref_cols='50',
	            dtpref_rows='20',REQUEST=None):
		"""
			Enables External DB support
		"""

		ret_value = NB_BaseContentDocument.inheritedAttribute("manage_edit")(self,data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST)
		
		self.updateContent()
		self.reindex_object()
		
		if ret_value:
			return ret_value
				
	def manage_upload(self,file='', REQUEST=None):
		"""
			Enables External DB support
		"""
		ret_value = NB_BaseContentDocument.inheritedAttribute("manage_upload")(self,file, REQUEST)
		
		self.updateContent()
		self.reindex_object()
		if ret_value:
			return ret_value

	def manage_afterAdd(self, item, container):
		""" 
			Called when objects get added, moved, or renamed.
		"""
		
		self.updateContent()

		NB_BaseContentDocument.inheritedAttribute("manage_afterAdd")(self, item, container)
			
	def manage_beforeDelete(self, item, container):
		""" 
			Called when objects get moved, cut-and-pasted, renamed, or deleted.
		"""		
		NB_BaseContentDocument.inheritedAttribute("manage_beforeDelete")(self, item, container)

class NB_BaseContentFolder(Folder, NB_BaseContentDocument):
	"""
		NeoBoard Base Content Folder Class for articles
	"""
	pass
	

class NB_AdvancedFolder(NB_BaseContentFolder):
	"""
		NeoBoard Advanced Folder Class
	"""

	def __init__(self):
		"""
			Content not cacheable by default
		"""
		self.setContentCacheable(0)

	def index_html(self):
		"""
			Returns the index page only if the site is enabled
		"""
		
		return self.getThemeObject_NB(id='index_html', obj_type='templates')
		

	def getNeoBoardThemeFolder_NB(self):
		"""
			Returns the first element from list of available NeoBoard Theme Folders
		"""	
		try:
			return self.superValues(('NeoBoard Theme Folder',))[0]
			
		except:
			return None

	
	def getThemeID_NB(self):
		"""
			Returns the ID of the current theme
		"""
		
		if hasattr(self, 'theme'):
			return self.theme
		else:
			return None

	def setTheme(self, theme):
		"""
			Sets the current theme to the named one
		"""	
		self.theme = theme
	
	def getDefaultTheme_NB(self):
		return getattr(self.getNeoBoardThemeFolder_NB(), 'default')

	def getThemeObject_NB(self, id, obj_type=None, css_class=None, render=1):
		""" 
			Returns the requested theme object
			obj_type - None: objects in the main folder
			obj_type - icons: objects in icons folder
			obj_type - images: objects in images folder
			obj_type - templates: objects in templates folder
		"""
		
		if hasattr(self, 'getTheme_NB'):
			theme = self.getTheme_NB()
		else:
			theme = self.getDefaultTheme_NB()
		
		if theme:
			obj = theme._getThemeObject(id, obj_type, css_class, render)
			if obj:
				if obj_type == 'templates' and render:
					return apply(obj, (self, self.REQUEST))
				else: 
					return obj
			else:
				obj = None
				for t in theme.getBackupThemes():
					try:
						obj = self.getTheme_NB(theme=t)._getThemeObject(id, obj_type, css_class, render)
					except:
						pass
					if obj: 
						if obj_type == 'templates' and render:
							return apply(obj, (self, self.REQUEST))
						else:
							return obj

		theme = self.getDefaultTheme_NB()
		obj = theme._getThemeObject(id, obj_type, css_class, render)
		if obj:
			if obj_type == 'templates' and render:
				return apply(obj, (self, self.REQUEST))
			else:
				return obj
				
		raise 'NeoBoard Theme Error', "The default theme <em><b>%s</b></em> doesn't have an attribute '%s' in folder '%s'!" % (theme.id, id, obj_type)		

	def getThemeObjectURL(self, id, obj_type=None):
		return self.getThemeObject_NB(id=id, obj_type=obj_type, render=0).absolute_url()

	def getThemes_NB(self):
		"""
			Returns the list of available themes
		"""
		return self.getNeoBoardThemeFolder_NB().getThemes_NB()

			
	def getTheme_NB(self, theme=None):
		""" 
			Returns the current theme
		"""
		
		if theme and hasattr(self.getNeoBoardThemeFolder_NB(), theme) and self.getNeoBoardThemeFolder_NB()[theme].isEnabled():
			try:
				return getattr(self.getNeoBoardThemeFolder_NB(), theme)
			except:
				return None
				
		if hasattr(self.getNeoBoardThemeFolder_NB(), self.getThemeID_NB()) and self.getNeoBoardThemeFolder_NB()[self.getThemeID_NB()].isEnabled():
			return getattr(self.getNeoBoardThemeFolder_NB(), self.getThemeID_NB())
			
		return self.getDefaultTheme_NB()

	def getPageLimit(self):
		if hasattr(self, 'page_limit'):
			return self.page_limit
		else:
			return 0

	def getMessage_NB(self, msg):
		""" 
			Returns the translated version of the named message string. (Wrapper method that calls the corresponding method in NB_Theme)
		"""	
		return self.getTheme_NB()._getMessage(msg)
		
	def _reRenderAll(self):		
		""" 
			Re-renders all renderable objects the container holds when renderable formats are changed
		"""			
		pass
			
class NB_Human(SimpleItem):

	delimiter = '__S__'

	def __init__(self, username='', realname='', email=''):
		self.username=username
		self.realname=realname
		self.email=email

	def getUserName(self):
		"""
			Returns Username
		"""
		if self.username:
			return self.username
		else:
			return ''
	
	def getRealName(self):
		"""
			Returns Real Name
		"""	
		if self.realname:
			return self.realname
		else:
			return ''		
	
	def getEmail(self):
		"""
			Returns email
		"""	
		if self.email:
			return self.email	
		else:
			return ''
	
	def getDelimiter(self):
		"""
			Returns delimiter
		"""	
		return self.delimiter
	
	def __str__(self):
		"""
			Returns string representation of this object
		"""	
		return self.getUserName() + self.getDelimiter() +  self.getRealName() + self.getDelimiter() + self.getEmail()